/**
  ******************************************************************************
  * File Name          : main.c
  * Description        : Main program body
  ******************************************************************************
  *
  * COPYRIGHT(c) 2017 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "stm32f0xx_hal.h"


/* USER CODE BEGIN Includes */

#define OSC_FREQUENCY 19200000
#define FINE_TUNE_INTERVAL 8
#define COARSE_TUNE_INTERVAL 1
#define INITIAL_TUNE_INTERVAL 0
#define FINE_MATCHES_NUM 5
/* amount of cycles to compensate, they lost during resetting of counter (with interrupt optimization). */
//#define PROCESSOR_DELAY_CYCLES 87+5
//orig:
//#define PROCESSOR_DELAY_CYCLES 54
#define PROCESSOR_DELAY_CYCLES 0


#define DISABLE_PLL 1
//#define DISABLE_FLASH_SAVING 1

/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/
TIM_HandleTypeDef htim2;
TIM_HandleTypeDef htim16;

UART_HandleTypeDef huart1;

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
void Error_Handler(void);
static void MX_GPIO_Init(void);
static void MX_TIM2_Init(void);
static void MX_TIM16_Init(void);
static void MX_USART1_UART_Init(void);

void HAL_TIM_MspPostInit(TIM_HandleTypeDef *htim);

uint8_t rx_buf[RXBUF_LEN], tx_buf[TXBUF_LEN];
/* xx_i - counter of input bytes (tx - pushed for transmit, rx - received)
   xx_o - counter of output bytes (tx - transmitted, rx - parsed)
   xx_e - counter of echoed bytes */
volatile uint16_t rx_i = 0, tx_o = 0;
uint16_t rx_o = 0, rx_e = 0, tx_i = 0;
volatile uint8_t tx_busy = 0;

void transmit(uint8_t byte)
{
    tx_buf[TXBUF_MSK & tx_i] = byte;
    tx_i++;
    tx_busy = 1;
    __HAL_UART_ENABLE_IT(&huart1, UART_IT_TXE);
}

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/
#ifndef abs
#define abs(x) (x >= 0 ? x : -x)
#endif
/* USER CODE END PFP */

/* USER CODE BEGIN 0 */
#ifndef DEBUG_TO_UART
extern void initialise_monitor_handles(void);	/*rtt*/
#endif
//#include <stdio.h>

/* You need this if you want use printf
 * https://github.com/mrsin/k-lab/wiki/STM32-%D0%BF%D1%80%D0%BE%D1%88%D0%B8%D0%B2%D0%BA%D0%B0-%D1%81-%D0%BF%D0%BE%D0%BC%D0%BE%D1%89%D1%8C%D1%8E-OpenOCD-%D0%B8-%D0%BE%D1%82%D0%BB%D0%B0%D0%B4%D0%BA%D0%B0-%D0%B2-Eclipse
 *  */
#ifdef DEBUG_TO_UART
int _write(int file, char *data, int len)
{
   //HAL_UART_Transmit(&huart1, (uint8_t*)data, len, len*100);
#if 1
   int index;
   /* For example, output string by UART */
   for(index=0; index<len; index++)
   {
     transmit(data[index]);
   }
#endif
    return len;
}

int _write_r (struct _reent *r, int file, char * ptr, int len)
{
  r = r;
  file = file;
  ptr = ptr;
#if 0
   int index;
   /* For example, output string by UART */
   for(index=0; index<len; index++)
   {
     if (ptr[index] == '\n')
     {
    	 ITM_SendChar('\r');
     }
     ITM_SendChar(ptr[index]);
   }
#else
  //HAL_UART_Transmit(&huart1, (uint8_t*)ptr, len, len*100);
#if 1
   int index;
   /* For example, output string by UART */
   for(index=0; index<len; index++)
   {
     transmit(ptr[index]);
   }
#endif
#endif
  return len;
}
#endif
/* USER CODE END 0 */


int  pwmout = 32767;
int  pwmout_old;
uint8_t  measures = 0;
uint8_t fine_tune_matches = 0;
uint32_t input_capture_accum;
uint32_t input_capture_accum_saved = 0;
uint32_t input_capture_accum_started = 0;
uint32_t input_capture_accum_1sec = 0;
uint32_t input_capture_accum_prev = 0;

int diff = 0;
int diff_1s = 0;
uint16_t step = 0;
uint8_t mode = COARSE_TUNE_INTERVAL;
int skip_1st=1;

//http://www.g4jnt.com/EvenSimplerSimplestGpsdoPossible.htm
//http://www.g4jnt.com/freqlock.htm

__attribute__( ( long_call, section(".ramfunctions") ) ) void HAL_TIM_IC_CaptureCallback(TIM_HandleTypeDef *htim)
{
	if (htim->Instance==TIM2)
		{
			/* this is done directly in interrupt too, can be removed from here */
			//if(measures == 0)
			//	__HAL_TIM_SetCounter(&htim2, 0); //<-exact place for processor delay
			input_capture_accum = __HAL_TIM_GetCompare(&htim2, TIM_CHANNEL_4);

			if(measures==0) input_capture_accum_started = input_capture_accum;

			measures++;

			input_capture_accum_1sec = input_capture_accum - input_capture_accum_prev;
			input_capture_accum_prev = input_capture_accum;
			diff_1s = OSC_FREQUENCY*COARSE_TUNE_INTERVAL - input_capture_accum_1sec;

			if(mode == INITIAL_TUNE_INTERVAL) {
				HAL_GPIO_WritePin(FREQ_ERR_MINUS_GPIO_Port, FREQ_ERR_MINUS_Pin, GPIO_PIN_RESET);
				HAL_GPIO_WritePin(FREQ_ERR_PLUS_GPIO_Port, FREQ_ERR_PLUS_Pin, GPIO_PIN_RESET);
				HAL_GPIO_WritePin(FREQ_CORRECT_GPIO_Port, FREQ_CORRECT_Pin, GPIO_PIN_RESET);
				mode = COARSE_TUNE_INTERVAL;
				measures=0;
				skip_1st=1;
				return;
			}

			if(mode == COARSE_TUNE_INTERVAL) {
				if(skip_1st==1) {
					skip_1st=0;
					return;
				}
				uint32_t abs_diff = abs(diff_1s);
				if(abs_diff > 128)
					step = 16;
				else if(abs_diff > 64)
					step = 8;
				else if(abs_diff > 8)
					step = 4;
				else if(abs_diff > 4)
					step = 2;
				else {
					step = 1;
#ifdef SEC8_MATCH
					if(abs_diff <= 2) {
						if(measures > FINE_MATCHES_NUM*8) {
							mode = FINE_TUNE_INTERVAL;
							measures=0;
							//fine_tune_matches = 0;
							fine_tune_matches = FINE_MATCHES_NUM-1;
						}
					} else {
						measures=0;
					}
#else
					if(abs_diff <= 2) {
						if(measures > FINE_MATCHES_NUM*8) {
							//mode = FINE_TUNE_INTERVAL;
							measures=0;
							//fine_tune_matches = 0;
							if(fine_tune_matches == 0) {
								fine_tune_matches = FINE_MATCHES_NUM+1;
								HAL_GPIO_WritePin(FREQ_CORRECT_GPIO_Port, FREQ_CORRECT_Pin, GPIO_PIN_SET);
								printf("$GPTXT,GPSDO_TUNED!*FF\r\n");
							}
						}
					} else {
						measures=0;
						if(fine_tune_matches == FINE_MATCHES_NUM+1 && abs_diff > 3) {
							fine_tune_matches = 0;
							HAL_GPIO_WritePin(FREQ_CORRECT_GPIO_Port, FREQ_CORRECT_Pin, GPIO_PIN_RESET);
							printf("$GPTXT,GPSDO_LOST!*FF\r\n");
						}
					}
#endif
				}
				pwmout += step * diff_1s;
				if (pwmout > 65535) pwmout=65535;
				if (pwmout < 0) pwmout=0;
				if(pwmout_old != pwmout) {
					__HAL_TIM_SetCompare(&htim16, TIM_CHANNEL_1, pwmout);
					pwmout_old = pwmout;
				}
			} else if(mode > COARSE_TUNE_INTERVAL && measures == mode+1) {
				input_capture_accum_saved = input_capture_accum - input_capture_accum_started;
				//printf("$GPTXT,pulses=%lu\r\n", input_capture_accum);
				//printf("$GPTXT,FREQ=%lu.%04lu\r\n", input_capture_accum/FINE_TUNE_INTERVAL, (input_capture_accum % (FINE_TUNE_INTERVAL))  * 625);
				//printf("FREQ=%lu\r\n", input_capture_accum);
				measures=0;
				diff = OSC_FREQUENCY*mode - input_capture_accum_saved;
				step = abs(diff);


				if(step > 1024)
					step = step*4;
				else if(step > 128)
					step = step*3;
				else if(step > 16)
					step = step*2;

				pwmout_old = pwmout;
				if(diff > 0) {
					pwmout += step;
					HAL_GPIO_WritePin(FREQ_ERR_PLUS_GPIO_Port, FREQ_ERR_PLUS_Pin, GPIO_PIN_SET);
					HAL_GPIO_WritePin(FREQ_ERR_MINUS_GPIO_Port, FREQ_ERR_MINUS_Pin, GPIO_PIN_RESET);
				} else if(diff < 0) {
					pwmout -= step;
					HAL_GPIO_WritePin(FREQ_ERR_PLUS_GPIO_Port, FREQ_ERR_PLUS_Pin, GPIO_PIN_RESET);
					HAL_GPIO_WritePin(FREQ_ERR_MINUS_GPIO_Port, FREQ_ERR_MINUS_Pin, GPIO_PIN_SET);
				}

				//if(diff != 0)
//				printf("$GPTXT,GPSDO_16SEC,%s,%lu,%lu.%04lu,%d,PWM,%d*FF\r\n",
//						diff > 0 ? "+" : "-",
//						input_capture_accum, input_capture_accum/FINE_TUNE_INTERVAL, (input_capture_accum % (FINE_TUNE_INTERVAL))  * 625,
//						step, pwmout);


				if(abs(diff) <= 10) {
					if(fine_tune_matches < FINE_MATCHES_NUM) {
						fine_tune_matches++;
					} else if(fine_tune_matches == FINE_MATCHES_NUM) {
						fine_tune_matches++;
						HAL_GPIO_WritePin(FREQ_CORRECT_GPIO_Port, FREQ_CORRECT_Pin, GPIO_PIN_SET);
						printf("$GPTXT,GPSDO_TUNED!*FF\r\n");
#ifndef DISABLE_FLASH_SAVING
						//can be written to FLASH!
						uint32_t *pwmout_sign_u32 =  (uint32_t*)0x8003C08;
  					   if(*pwmout_sign_u32 != 0xa5a5a5a5) {
  						  uint32_t SectorError = 0;
  						  FLASH_EraseInitTypeDef pEraseInit;
  						  pEraseInit.TypeErase = TYPEERASE_PAGES;
  						  pEraseInit.PageAddress = 0x8003C00;
  						  pEraseInit.NbPages = 1;

  						  HAL_FLASH_Unlock();
  						  if (	HAL_FLASHEx_Erase(&pEraseInit, &SectorError) != HAL_OK
  							 || HAL_FLASH_Program(TYPEPROGRAM_WORD, 0x8003C04, pwmout) != HAL_OK
							 || HAL_FLASH_Program(TYPEPROGRAM_WORD, 0x8003C08, 0xa5a5a5a5) != HAL_OK)
  							  printf("$GPTXT,PGMERR*FF\r\n");
  						  HAL_FLASH_Lock();
  						  printf("$GPTXT,FIRST FIX saved!*FF\r\n");
  					   }
#endif
					}
				} else if(abs(diff) > 10) {
					if(fine_tune_matches > FINE_MATCHES_NUM) {
						HAL_GPIO_WritePin(FREQ_CORRECT_GPIO_Port, FREQ_CORRECT_Pin, GPIO_PIN_RESET);
						printf("$GPTXT,GPSDO_LOST!*FF\r\n");
					}
					fine_tune_matches = 0;
				}

				if(abs(diff) > 0) {
					if (pwmout > 65535) pwmout=65535;
					if (pwmout < 0) pwmout=0;
					__HAL_TIM_SetCompare(&htim16, TIM_CHANNEL_1, pwmout);
				} else {
					 pwmout = pwmout_old;
				}

			}

		}
}

const char text_to_find[] = "\n$GPGLL";
//  \xb6\x62<CLASS><ID><LEN><PAYLOAD><CRC>
// where
//  <CLASS> is 1 byte
//  <ID>    is 1 byte
//  <LEN>   is 2 bytes (little-endian), length of <PAYLOAD>
//  <CRC>   is 2 bytes
const uint8_t sumulate_ubx_message_nolock[64] = {
	0xb5, 0x62, 0x0d, 0x12, 0x38, 0x00, 0x00, 0x00,  0x1f, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7c, 0xef
};

#if 0
const uint8_t sumulate_ubx_message_lock[64] = {
	0xb5, 0x62, 0x0d, 0x12, 0x38, 0x00, 0x00, 0x00,  0xd2, 0x00, 0xd8, 0x39, 0x00, 0x00, 0xe1, 0x07,
	0x01, 0x12, 0x0e, 0x10, 0x06, 0x03, 0xf9, 0xff,  0xff, 0xff, 0x08, 0x00, 0x00, 0x00, 0x8c, 0x07,
	0x00, 0x00, 0x38, 0xbd, 0x04, 0x00, 0xf9, 0xff,  0xff, 0xff, 0x08, 0x00, 0x00, 0x00, 0xd7, 0xff,
	0xff, 0xff, 0x3d, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf5, 0x07
}
#else
const uint8_t sumulate_ubx_message_lock[64] = {
	0xb5, 0x62, 0x0d, 0x12, 0x38, 0x00, 0x00, 0x00, 0x84, 0x00, 0xd8, 0x39, 0x00, 0x00, 0xe1, 0x07,
	0x01, 0x12, 0x0e, 0x1f, 0x34, 0x03, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x8c, 0x07,
	0x00, 0x00, 0xea, 0xc0, 0x04, 0x00, 0x05, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0xfe, 0xfe,
	0xff, 0xff, 0x33, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xca, 0x02
};
#endif

int main(void)
{

  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration----------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* Configure the system clock */
  SystemClock_Config();

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_USART1_UART_Init();
#ifndef DISABLE_FLASH_SAVING
  uint16_t *pwmout_u16 =  (uint16_t*)0x8003C04;
  uint32_t *pwmout_sign_u32 =  (uint32_t*)0x8003C08;

  if(*pwmout_sign_u32 != 0xa5a5a5a5) {
	  pwmout = 0x7FFF;
	  printf("$GPTXT,LDFLASH,0*FF\r\n");
  } else {
	  pwmout = *pwmout_u16;
	  printf("$GPTXT,LDFLASH,%u*FF\r\n", (uint16_t) (*pwmout_u16));
	  if(pwmout == 0xFFFF) pwmout = 0x7FFF;
  }
#endif
  //after readed EEPROM value!
  MX_TIM16_Init();
  MX_TIM2_Init();

  /* USER CODE BEGIN 2 */
  HAL_TIM_PWM_Start(&htim16, TIM_CHANNEL_1);
  //HAL_TIM_Base_Start(&htim2);
  HAL_TIM_IC_Start_IT(&htim2, TIM_CHANNEL_4);
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */


  uint8_t next_is_cmd = 0;
  uint8_t have_NMEA=0;
  /* Enable usart 1 receive IRQ */
  __HAL_UART_ENABLE_IT(&huart1, UART_IT_RXNE);
  for (;;) {
#if 1
      /* Main cycle */
      while (rx_i != rx_e) {
          /* echo here */
          transmit(rx_buf[RXBUF_MSK & rx_e]);
          //transmit(next_is_cmd+0x30);

          switch(next_is_cmd) {
        	case 3: if(rx_buf[rx_e & RXBUF_MSK] == 'N') rx_buf[rx_e & RXBUF_MSK] = 'P';
        	case 0:
          	case 1:
        	case 2:
          	case 4:
          	case 5:
          	case 6:
          		if(rx_buf[rx_e & RXBUF_MSK] == text_to_find[next_is_cmd]) next_is_cmd++; else next_is_cmd=0; break;
          	//case 7: if(rx_buf[rx_e & RXBUF_MSK] == '\r') next_is_cmd++; break;
          	case 7:
          			if(rx_buf[rx_e & RXBUF_MSK] == '\n') {

          				if(measures == 0) {
          					printf("$GPTXT,GPSDO_%dSEC,%s,%lu,%lu.%04lu,%d,PWM,%d*FF\r\n", mode,
          						diff > 0 ? "+" : "-",
          								input_capture_accum_saved,
										input_capture_accum_saved/mode, (input_capture_accum_saved % (mode))  * 10000/mode,
          						step, pwmout);

          				}
          				const uint8_t *to_transmit = sumulate_ubx_message_nolock;
      					if(fine_tune_matches == FINE_MATCHES_NUM+1) to_transmit = sumulate_ubx_message_lock;

          				do {
          					transmit(to_transmit[next_is_cmd-8]);
          					next_is_cmd++;
          				} while (next_is_cmd < 8+64);
          				next_is_cmd=0;
          				printf("$GPTXT,GPSDO_1SEC,%lu,D,%d,PWM,%d*FF\r\n", input_capture_accum_1sec, diff_1s, pwmout);
          				//input_capture_accum_prev = input_capture_accum;
          			}
          			have_NMEA=1;
          			break;
          	}
          rx_e++;
      }
      if(have_NMEA==0) {
			if(measures == 0) {
				printf("$GPTXT,GPSDO_%dSEC,%s,%lu,%lu.%04lu,%d,PWM,%d*FF\r\n", mode,
					diff > 0 ? "+" : "-",
							input_capture_accum_saved,
							input_capture_accum_saved/mode, (input_capture_accum_saved % (mode))  * 10000/mode,
					step, pwmout);

			}

			//printf("$GPTXT,GPSDO_1SEC,%lu*FF\r\n", input_capture_accum - input_capture_accum_prev);
			printf("$GPTXT,GPSDO_1SEC,%lu,D,%d,PWM,%d*FF\r\n", input_capture_accum_1sec, diff_1s, pwmout);
			//input_capture_accum_prev = input_capture_accum;
			HAL_Delay(1000);
  	  }
#endif
      /* Power save
      while (tx_busy);
      HAL_UART_DeInit(&huart1);
      */
  }
  /* USER CODE END 3 */

}

/** System Clock Configuration
*/
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_PeriphCLKInitTypeDef PeriphClkInit;

  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;

  //TODO: unstable!!!
  //RCC_OscInitStruct.HSEState = RCC_HSE_BYPASS;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;

#ifdef DISABLE_PLL
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
#else
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  //RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL2;
  //RCC_OscInitStruct.PLL.PREDIV = RCC_PREDIV_DIV1;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL5;
  RCC_OscInitStruct.PLL.PREDIV = RCC_PREDIV_DIV2;
#endif
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
#ifdef DISABLE_PLL
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSE;
  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
#else
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK)
#endif
  {
    Error_Handler();
  }

  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART1;
  PeriphClkInit.Usart1ClockSelection = RCC_USART1CLKSOURCE_PCLK1;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }

  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 1, 0);
}

/* TIM2 init function */
static void MX_TIM2_Init(void)
{

  TIM_ClockConfigTypeDef sClockSourceConfig;
  TIM_MasterConfigTypeDef sMasterConfig;
  TIM_IC_InitTypeDef sConfigIC;

  htim2.Instance = TIM2;
  htim2.Init.Prescaler = 0;
  htim2.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim2.Init.Period = 0xFFFFFFFF;
  htim2.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  if (HAL_TIM_Base_Init(&htim2) != HAL_OK)
  {
    Error_Handler();
  }
#ifdef DISABLE_PLL
  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
#else
  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_ETRMODE2;
  sClockSourceConfig.ClockFilter = 0;
#endif
  sClockSourceConfig.ClockPolarity = TIM_CLOCKPOLARITY_NONINVERTED;
  sClockSourceConfig.ClockPrescaler = TIM_CLOCKPRESCALER_DIV1;
  if (HAL_TIM_ConfigClockSource(&htim2, &sClockSourceConfig) != HAL_OK)
  {
    Error_Handler();
  }

  if (HAL_TIM_IC_Init(&htim2) != HAL_OK)
  {
    Error_Handler();
  }

  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim2, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }

  sConfigIC.ICPolarity = TIM_INPUTCHANNELPOLARITY_RISING;
  sConfigIC.ICSelection = TIM_ICSELECTION_DIRECTTI;
  sConfigIC.ICPrescaler = TIM_ICPSC_DIV1;
  //sConfigIC.ICFilter = 15;
  sConfigIC.ICFilter = 0;
  if (HAL_TIM_IC_ConfigChannel(&htim2, &sConfigIC, TIM_CHANNEL_4) != HAL_OK)
  {
    Error_Handler();
  }

}

/* TIM16 init function */
static void MX_TIM16_Init(void)
{

  TIM_BreakDeadTimeConfigTypeDef sBreakDeadTimeConfig;
  TIM_OC_InitTypeDef sConfigOC;

  /*
   * Determine the desired PWM resolution (for example: 100 steps, 200 steps, 1000…)
   * Determine the desired PWM frequency (for example: 1kHz, 10kHz, 40kHz, …)
   * Set the Counter Period at the same value as the resolution
   * Set the Prescaler = 48000000/(PWM frequency*PWM resolution) – 1 (thanks to Ruben and Avinash to spot out the mistake)
   * Set the Pulse equal to the desired Pulse width where the value varies from 0 to Counter Period.
   *
   * */
  htim16.Instance = TIM16;
  //htim16.Init.Prescaler = (48000000/(10000*200) - 1);
  htim16.Init.Prescaler = (19200000/(146 * 65535) - 1);
  htim16.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim16.Init.Period = 65535;
  htim16.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim16.Init.RepetitionCounter = 0;
  if (HAL_TIM_Base_Init(&htim16) != HAL_OK)
  {
    Error_Handler();
  }

  if (HAL_TIM_PWM_Init(&htim16) != HAL_OK)
  {
    Error_Handler();
  }

  sBreakDeadTimeConfig.OffStateRunMode = TIM_OSSR_DISABLE;
  sBreakDeadTimeConfig.OffStateIDLEMode = TIM_OSSI_DISABLE;
  sBreakDeadTimeConfig.LockLevel = TIM_LOCKLEVEL_OFF;
  sBreakDeadTimeConfig.DeadTime = 0;
  sBreakDeadTimeConfig.BreakState = TIM_BREAK_DISABLE;
  sBreakDeadTimeConfig.BreakPolarity = TIM_BREAKPOLARITY_HIGH;
  sBreakDeadTimeConfig.AutomaticOutput = TIM_AUTOMATICOUTPUT_DISABLE;
  if (HAL_TIMEx_ConfigBreakDeadTime(&htim16, &sBreakDeadTimeConfig) != HAL_OK)
  {
    Error_Handler();
  }

  sConfigOC.OCMode = TIM_OCMODE_PWM1;
  sConfigOC.Pulse = pwmout;
  sConfigOC.OCPolarity = TIM_OCPOLARITY_HIGH;
  sConfigOC.OCNPolarity = TIM_OCNPOLARITY_HIGH;
  sConfigOC.OCFastMode = TIM_OCFAST_DISABLE;
  sConfigOC.OCIdleState = TIM_OCIDLESTATE_RESET;
  sConfigOC.OCNIdleState = TIM_OCNIDLESTATE_RESET;
  if (HAL_TIM_PWM_ConfigChannel(&htim16, &sConfigOC, TIM_CHANNEL_1) != HAL_OK)
  {
    Error_Handler();
  }

  HAL_TIM_MspPostInit(&htim16);

}

/* USART1 init function */
static void MX_USART1_UART_Init(void)
{

  huart1.Instance = USART1;
  huart1.Init.BaudRate = 9600;
  huart1.Init.WordLength = UART_WORDLENGTH_8B;
  huart1.Init.StopBits = UART_STOPBITS_1;
  huart1.Init.Parity = UART_PARITY_NONE;
  huart1.Init.Mode = UART_MODE_TX_RX;
  huart1.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart1.Init.OverSampling = UART_OVERSAMPLING_16;
  huart1.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart1.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart1) != HAL_OK)
  {
    Error_Handler();
  }

}

/** Configure pins as 
        * Analog 
        * Input 
        * Output
        * EVENT_OUT
        * EXTI
*/
static void MX_GPIO_Init(void)
{

  GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOF_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pins : PA0 PA1 PA2 PA4 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_2|GPIO_PIN_4;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, FREQ_ERR_MINUS_Pin|FREQ_CORRECT_Pin, GPIO_PIN_SET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(FREQ_ERR_PLUS_GPIO_Port, FREQ_ERR_PLUS_Pin, GPIO_PIN_SET);

  /*Configure GPIO pins : FREQ_ERR_MINUS_Pin FREQ_CORRECT_Pin */
  GPIO_InitStruct.Pin = FREQ_ERR_MINUS_Pin|FREQ_CORRECT_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : FREQ_ERR_PLUS_Pin */
  GPIO_InitStruct.Pin = FREQ_ERR_PLUS_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(FREQ_ERR_PLUS_GPIO_Port, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
  }
  /* USER CODE END Error_Handler */ 
}

#ifdef USE_FULL_ASSERT

/**
   * @brief Reports the name of the source file and the source line number
   * where the assert_param error has occurred.
   * @param file: pointer to the source file name
   * @param line: assert_param error line source number
   * @retval None
   */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
    ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */

}

#endif

/**
  * @}
  */ 

/**
  * @}
*/ 

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
